"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultOCABundleResolver = exports.OCABundle = exports.BrandingOverlayType = void 0;
const lodash_startcase_1 = __importDefault(require("lodash.startcase"));
const constants_1 = require("../../constants");
const types_1 = require("../../types");
const color_1 = require("../../utils/color");
const credential_definition_1 = require("../../utils/credential-definition");
var BrandingOverlayType;
(function (BrandingOverlayType) {
    BrandingOverlayType["Branding01"] = "aries/overlays/branding/0.1";
    BrandingOverlayType["Branding10"] = "aries/overlays/branding/1.0";
})(BrandingOverlayType || (exports.BrandingOverlayType = BrandingOverlayType = {}));
class OCABundle {
    constructor(bundle, options) {
        this.bundle = bundle;
        this.options = {
            brandingOverlayType: options?.brandingOverlayType ?? BrandingOverlayType.Branding10,
            language: options?.language ?? constants_1.defaultBundleLanguage,
        };
    }
    get captureBase() {
        const overlay = this.bundle.captureBase;
        if (!overlay) {
            throw new Error('Capture Base must be defined');
        }
        return overlay;
    }
    get characterEncodingOverlay() {
        return this.getOverlay(types_1.OverlayType.CharacterEncoding10);
    }
    get formatOverlay() {
        return this.getOverlay(types_1.OverlayType.Format10);
    }
    get labelOverlay() {
        return this.getOverlay(types_1.OverlayType.Label10, this.options.language);
    }
    get metaOverlay() {
        return this.getOverlay(types_1.OverlayType.Meta10, this.options.language);
    }
    get brandingOverlay() {
        return this.getOverlay(this.options?.brandingOverlayType || BrandingOverlayType.Branding10);
    }
    buildOverlay(name, language) {
        return new types_1.MetaOverlay({
            capture_base: '',
            type: types_1.OverlayType.Meta10,
            name,
            language,
            description: '',
            credential_help_text: '',
            credential_support_url: '',
            issuer: '',
            issuer_description: '',
            issuer_url: '',
        });
    }
    getOverlay(type, language) {
        if (type === types_1.OverlayType.CaptureBase10) {
            return this.bundle.captureBase;
        }
        if (language !== undefined) {
            // we want to return branding even if there isn't a bundle for a given language
            const overlay = this.bundle.overlays.find((item) => (item.language === undefined && item.type === type.toString()) ||
                (item.type === type.toString() && item.language === language));
            if (overlay) {
                return overlay;
            }
        }
        return this.bundle.overlays.find((item) => item.type === type.toString());
    }
}
exports.OCABundle = OCABundle;
class DefaultOCABundleResolver {
    constructor(bundlesData = {}, options) {
        this.bundles = {};
        for (const cid in bundlesData) {
            try {
                if (typeof bundlesData[cid] !== 'string') {
                    this.bundles[cid] = new types_1.OverlayBundle(cid, bundlesData[cid]);
                }
                else {
                    this.bundles[cid] = bundlesData[cid];
                }
            }
            catch (error) {
                // might get an error trying to parse javascript's default value
                this.log?.error(`Error parsing bundle for ${cid}`, error);
            }
        }
        this.options = {
            brandingOverlayType: options?.brandingOverlayType ?? BrandingOverlayType.Branding10,
            language: options?.language ?? constants_1.defaultBundleLanguage,
        };
    }
    /**
     * Sets the log value.
     * @param value - The new value for the log.
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    set log(value) {
        this._log = value;
    }
    /**
     * Get the log value.
     */
    get log() {
        return this._log;
    }
    getBrandingOverlayType() {
        return this.options.brandingOverlayType ?? BrandingOverlayType.Branding10;
    }
    getDefaultBundle(params) {
        if (!params.language) {
            params.language = constants_1.defaultBundleLanguage;
        }
        const metaOverlay = {
            capture_base: '',
            type: types_1.OverlayType.Meta10,
            name: (0, lodash_startcase_1.default)(params.meta?.credName ??
                (0, credential_definition_1.parseCredDefFromId)(params.identifiers?.credentialDefinitionId, params.identifiers?.schemaId)),
            issuer: params.meta?.alias || params.meta?.credConnectionId || 'Unknown Contact',
            language: params.language ?? this.options?.language,
            description: '',
            credential_help_text: '',
            credential_support_url: '',
            issuer_description: '',
            issuer_url: '',
        };
        let colorHash = 'default';
        if (metaOverlay?.name) {
            colorHash = metaOverlay.name;
        }
        else if (metaOverlay?.issuer) {
            colorHash = metaOverlay.issuer;
        }
        const brandingoOverlay01 = {
            capture_base: '',
            type: types_1.OverlayType.Branding01,
            background_color: (0, color_1.generateColor)(colorHash),
        };
        const brandingoOverlay10 = {
            capture_base: '',
            type: types_1.OverlayType.Branding10,
            primary_background_color: (0, color_1.generateColor)(colorHash),
        };
        const bundle = new types_1.OverlayBundle(params.identifiers?.credentialDefinitionId, {
            capture_base: {
                attributes: {},
                classification: '',
                type: types_1.OverlayType.CaptureBase10,
                flagged_attributes: [],
            },
            overlays: [
                metaOverlay,
                this.getBrandingOverlayType() === BrandingOverlayType.Branding01 ? brandingoOverlay01 : brandingoOverlay10,
            ],
        });
        return Promise.resolve(new OCABundle(bundle, { ...this.options, language: params.language ?? this.options.language }));
    }
    resolveDefaultBundle(params) {
        return this.getDefaultBundle(params);
    }
    resolve(params) {
        const language = params.language || constants_1.defaultBundleLanguage;
        for (const item of [
            params.identifiers?.credentialDefinitionId,
            params.identifiers?.schemaId,
            params.identifiers?.templateId,
        ]) {
            if (item && this.bundles[item] !== undefined) {
                let bundle = this.bundles[item];
                // if it is a string, it is a reference/alias to another one bundle
                if (typeof bundle === 'string') {
                    bundle = this.bundles[bundle];
                }
                return Promise.resolve(new OCABundle(bundle, { ...this.options, language: language ?? this.options.language }));
            }
        }
        return Promise.resolve(undefined);
    }
    async presentationFields(params) {
        const bundle = await this.resolve(params);
        let presentationFields = [...params.attributes];
        if (bundle?.captureBase?.attributes) {
            // if the oca branding has the attributes set, only display those attributes
            const bundleFields = Object.keys(bundle.captureBase.attributes);
            presentationFields = presentationFields.filter((item) => item.name && bundleFields.includes(item.name));
            for (let i = 0; i < presentationFields.length; i++) {
                const presentationField = presentationFields[i];
                const key = presentationField.name || '';
                if (bundle.captureBase.attributes[key]) {
                    presentationField.label = bundle?.labelOverlay?.attributeLabels[key];
                    presentationField.format = bundle?.formatOverlay?.attributeFormats[key];
                    presentationField.type = bundle?.captureBase?.attributes?.[key];
                    presentationField.encoding = bundle?.characterEncodingOverlay?.attributeCharacterEncoding?.[key];
                }
            }
        }
        return presentationFields;
    }
    async resolveAllBundles(params) {
        const [bundle, defaultBundle] = await Promise.all([this.resolve(params), this.resolveDefaultBundle(params)]);
        const fields = params.attributes
            ? await this.presentationFields({
                ...params,
                attributes: params.attributes,
            })
            : [];
        const overlayBundle = bundle ?? defaultBundle;
        const metaOverlay = overlayBundle?.metaOverlay;
        const brandingOverlay = overlayBundle?.brandingOverlay;
        return { bundle: overlayBundle, presentationFields: fields, metaOverlay, brandingOverlay };
    }
}
exports.DefaultOCABundleResolver = DefaultOCABundleResolver;
