import { DefaultOCABundleResolver, Identifiers, OCABundle, OCABundleResolverOptions } from './oca';
export interface RemoteOCABundleResolverOptions extends OCABundleResolverOptions {
    indexFileName?: string;
    preLoad?: boolean;
}
type BundleIndexEntry = {
    path: string;
    sha256: string;
};
type BundleIndex = {
    [key: string]: BundleIndexEntry;
};
export declare class RemoteOCABundleResolver extends DefaultOCABundleResolver {
    protected indexFile: BundleIndex;
    private axiosInstance;
    private cacheDataFileName;
    private indexFileName;
    private _indexFileEtag?;
    constructor(indexFileBaseUrl: string, options?: RemoteOCABundleResolverOptions);
    /**
     * Sets the value of the index file ETag.
     *
     * @param value - The new value for the index file ETag.
     */
    set indexFileEtag(value: string);
    /**
     * Gets the ETag of the index file.
     *
     * @returns The ETag of the index file, or an empty string if not available.
     */
    get indexFileEtag(): string;
    /**
     * Checks for updates in the OCA (Overlay Capture Architecture) index.
     * If the index file ETag is not available, it loads the cache data and retrieves the ETag from it.
     * Then, it loads the OCA index.
     * @returns A promise that resolves when the update check is complete.
     */
    checkForUpdates(): Promise<void>;
    /**
     * Loads the cache data from the local storage.
     * @returns A promise that resolves to the cache data file, or undefined if the cache file does not exist or cannot be loaded.
     */
    private loadCacheData;
    /**
     * Saves the cache data to local storage.
     *
     * @param cacheData - The cache data to be saved.
     * @returns A promise that resolves to a boolean indicating whether the save operation was successful.
     */
    private saveCacheData;
    /**
     * Processes the queue of OCABundleQueueEntry items.
     *
     * @param items - An array of OCABundleQueueEntry items to process.
     * @returns A promise that resolves to an array of processed OCABundleQueueEntry items.
     */
    private processQueue;
    /**
     * Prepares the bundle queue based on the new and old index files.
     * It compares the SHA256 hashes of the files in the new and old index files
     * and determines which files should be removed and which files should be added
     * to the bundle queue.
     *
     * @param newIndexFile - The new bundle index file.
     * @param oldIndexFile - The old bundle index file.
     * @returns An array of `OCABundleQueueEntry` objects representing the files to be removed and added.
     */
    private prepareBundleQueue;
    /**
     * Finds the SHA-256 hash associated with a given path in the index file.
     *
     * @param {string} path - The path to search for in the index file.
     * @returns {string | null} The SHA-256 hash associated with the path if found, or null if not found.
     */
    private findSha256ByPath;
    /**
     * Finds the path associated with the given SHA256 hash in the index file.
     *
     * @param sha256 - The SHA256 hash to search for.
     * @returns The path associated with the SHA256 hash, or null if not found.
     */
    private findPathBySha256;
    /**
     * Returns the file name for the bundle at the specified path.
     *
     * @param path - The path of the bundle.
     * @returns The file name for the bundle, or null if not found.
     */
    private fileNameForBundleAtPath;
    /**
     * Returns the file storage path for the remote OCA resolver.
     * The file storage path is the concatenation of the `CachesDirectoryPath` and `ocaBundleStorageDirectory`.
     *
     * @returns The file storage path.
     */
    private fileStoragePath;
    /**
     * Checks if a file exists at a specific path in the document directory.
     *
     * @param {string} fileName - The name of the file to check.
     * @returns {Promise<boolean>} A promise that resolves to true if the file exists, or false otherwise.
     * @throws Will throw an error if the existence check fails.
     */
    private checkFileExists;
    /**
     * Creates a working directory if it does not already exist.
     *
     * @returns A promise that resolves to a boolean indicating whether the directory was created successfully.
     */
    private createWorkingDirectoryIfNotExists;
    /**
     * Saves a string of data to a file in the local storage.
     *
     * @param {string} fileName - The name of the file to save.
     * @param {string} data - The data to write to the file.
     * @returns {Promise<boolean>} A promise that resolves to true if the file was saved successfully, or false otherwise.
     * @throws Will throw an error if the write operation fails.
     */
    private saveFileToLocalStorage;
    /**
     * Loads a file from local storage.
     *
     * @param fileName - The name of the file to load.
     * @returns A promise that resolves to the contents of the file, or undefined if the file does not exist.
     */
    private loadFileFromLocalStorage;
    /**
     * Removes a file from the local storage.
     *
     * @param fileName - The name of the file to be removed.
     * @returns A promise that resolves to a boolean indicating whether the file was successfully removed.
     */
    private removeFileFromLocalStorage;
    /**
     * Fetches an OCA bundle from a remote resource and saves it to local storage.
     * @param path - The path of the remote resource.
     * @returns A promise that resolves to a boolean indicating whether the fetch and save operation was successful.
     */
    private fetchOCABundle;
    /**
     * Loads the OCA index from a remote location and processes it.
     * If the remote resource is not available, it falls back to the cached index file.
     * If the index file has not changed, it uses the existing data.
     * If the index file has changed, it refreshes the index file and the bundles.
     *
     * @param filePath - The path to the remote OCA index file.
     * @returns A Promise that resolves when the index file and bundles have been processed.
     * @throws If there is an error fetching or processing the OCA index.
     */
    private loadOCAIndex;
    /**
     * Loads the OCABundle from the specified path.
     *
     * @param path - The path of the OCABundle.
     * @returns A promise that resolves to an array of IOverlayBundleData.
     */
    private loadOCABundle;
    /**
     * Finds a matching identifier in the index file based on the provided identifiers.
     * The order of the identifiers matters if more than one candidate exists in the index file.
     *
     * @param identifiers - The identifiers to match against the index file.
     * @returns The matching identifier, or undefined if no match is found.
     */
    private matchBundleIndexEntry;
    /**
     * Resolves the OCABundle based on the given parameters.
     * @param params - The parameters for resolving the OCABundle.
     * @param params.identifiers - The identifiers used to match the OCABundle.
     * @param params.language - The language of the OCABundle (optional).
     * @returns A Promise that resolves to the OCABundle or undefined.
     */
    resolve(params: {
        identifiers: Identifiers;
        language?: string | undefined;
    }): Promise<OCABundle | undefined>;
    /**
     * Retrieves the fallback branding overlays for a given credential definition ID and capture base.
     * @param credentialDefinitionId - The ID of the credential definition.
     * @param captureBase - The capture base for the overlays.
     * @returns An array of fallback branding overlays.
     */
    private getFallbackBrandingOverlays;
}
export {};
